#exonware/xwsystem/tests/0.core/utils/__init__.py
#exonware/xwsystem/tests/core/utils/__init__.py
"""
Utils Core Tests Package
Tests for xwsystem utility functions including lazy loading,
path utilities, and common utilities.
"""
