#exonware/xwsystem/tests/0.core/performance/__init__.py
#exonware/xwsystem/tests/core/performance/__init__.py
"""
Performance Core Tests Package
Tests for xwsystem performance management including performance monitoring,
optimization, and resource management.
"""
